
--We store our saved variable table in here.
local rust = {}
rust.mappings = {}
--Add some default settings.
rust.settings = {}

local hasAlreadyLoaded = false

--Get addon name.
local addonName = Inspect.Addon.Detail(Inspect.Addon.Current())["name"]
--Player name and shard.
local playerName = Inspect.Unit.Detail("player").name .. "@" .. Inspect.Shard().name
local currentProfile = nil --We set this in the main() function, after variables and such have loaded.
--Configuration window.
rust.settings.configWindow = {}
rust.settings.configWindow.mainWindowHeight = 600
rust.settings.configWindow.mainWindowWidth = 800
rust.settings.configWindow.mainWindowLayer = 0
rust.settings.configWindow.mainWindowTitle = addonName
rust.settings.configWindow.mainWindowXOffset = 0
rust.settings.configWindow.mainWindowYOffset = 0
rust.settings.configWindow.mainWindowVisible = true

--Action bar.
rust.settings.actionBar = {}
rust.settings.actionBar.defaults = {}
rust.settings.actionBar.action_bars = {}

rust.settings.actionBar.defaults.height = 30
rust.settings.actionBar.defaults.width = 30
rust.settings.actionBar.defaults.proportionalWidthHeight = true

rust.settings.actionBar.defaults.horizontalSpacing = 2
rust.settings.actionBar.defaults.verticalSpacing = 2

rust.settings.actionBar.defaults.horizontalEdgePadding = 2
rust.settings.actionBar.defaults.verticalEdgePadding = 2

rust.settings.actionBar.defaults.icon = "Data/\\UI\\ability_icons\\macro_icon_no.dds"

rust.settings.actionBar.defaults.barOuterBackground = 
{
	["r"] = 0,
	["g"] = 0,
	["b"] = 0,
	["a"] = 1,
}
rust.settings.actionBar.defaults.barInnerBackground = 
{
	["r"] = 1,
	["g"] = 1,
	["b"] = 1,
	["a"] = 0,
}

rust.settings.actionBar.defaults.buttonBackground = 
{
	["r"] = 0.5,
	["g"] = 0.5,
	["b"] = 0.5,
	["a"] = 0.25,
}

rust.settings.actionBar.defaults.rows = 1
rust.settings.actionBar.defaults.columns = 12

rust.settings.actionBar.defaults.xOffset = 0
rust.settings.actionBar.defaults.yOffset = 0

rust.settings.actionBar.defaults.locked = false
rust.settings.actionBar.defaults.dragFrameWidth = 8
rust.settings.actionBar.defaults.dragFrameHeight = 8
rust.settings.actionBar.defaults.dragFrameBackground = 
{
	["r"] = 0,
	["g"] = 1,
	["b"] = 0,
	["a"] = 0.5,
}

rust.settings.actionBar.defaults.deleteFrameButtonWidth = 8
rust.settings.actionBar.defaults.deleteFrameButtonHeight = 8
rust.settings.actionBar.defaults.deleteFrameButtonBackground = 
{
	["r"] = 1,
	["g"] = 0,
	["b"] = 0,
	["a"] = 0.5,
}

local barFrameNumber = 0


--Global content (parent frame-thing).
local contextName = addonName.."_context"
local context = UI.CreateContext(contextName)
context:SetSecureMode("restricted")



--Utility functions.
local function make_set(list)
	if not list then return end
	
	local set = {}
	
	for _, v in pairs(list) do
		set[v] = true
	end
	
	return set
end




local function createBar(rows, columns)
	--Returns a frame.
	
	--File scope local variable - tracks the number of bars made.
	barFrameNumber = barFrameNumber + 1
	rust.settings.actionBar.action_bars[currentProfile][barFrameNumber] = rust.settings.actionBar.action_bars[currentProfile][barFrameNumber] or {}
	local barTableRef = rust.settings.actionBar.action_bars[currentProfile][barFrameNumber]
	
	if not next(barTableRef) then
		--table is empty - we are therefore creating a new table rather than restoring an old one.
		barTableRef.rows = tonumber(rows)
		barTableRef.columns = tonumber(columns)
	end
	
	setmetatable(barTableRef, {__index = rust.settings.actionBar.defaults})
	
	--Calculate the dimension of the frame.
	--local numberOfRowsColumns = math.ceil(numberOfActions / actionsPerRowColumn)
	--local primaryDimension, secondaryDimension, primaryButtonSize, secondaryButtonSize, primaryEdgePadding, secondaryEdgePadding, primarySpacing, secondarySpacing
	
	--if primaryOrientaton == "
	
	
	--Create our outer frame for the bar - we have an inner frame which buttons are anchored to.
	--This is to allow the button anchoring code to ignore bar edge padding.
	local barFrameOuter = UI.CreateFrame("Frame", context:GetName().."_barFrameOuter_"..barFrameNumber, context)
	barFrameOuter:SetBackgroundColor(barTableRef.barOuterBackground.r,
								barTableRef.barOuterBackground.g,
								barTableRef.barOuterBackground.b,
								barTableRef.barOuterBackground.a)
								
	barFrameOuter:SetWidth((barTableRef.columns * (barTableRef.width + barTableRef.horizontalSpacing)) + barTableRef.horizontalSpacing)
	barFrameOuter:SetHeight((barTableRef.rows * (barTableRef.height + barTableRef.verticalSpacing)) + barTableRef.verticalSpacing)
	
	barFrameOuter:SetSecureMode("restricted")
	
	--Create a drag frame for the bar.
	local dragFrame = UI.CreateFrame("Frame", barFrameOuter:GetName().."_dragFrame"..barFrameNumber, barFrameOuter)
	dragFrame:SetLayer(10)
	dragFrame:SetWidth(barTableRef.dragFrameWidth)
	dragFrame:SetHeight(barTableRef.dragFrameHeight)
	dragFrame:SetPoint("CENTER", UIParent, "CENTER", barTableRef.xOffset, barTableRef.yOffset)
	barFrameOuter:SetPoint("TOPLEFT", dragFrame, "TOPRIGHT") --
	dragFrame:SetBackgroundColor(barTableRef.dragFrameBackground.r,
								barTableRef.dragFrameBackground.g,
								barTableRef.dragFrameBackground.b,
								barTableRef.dragFrameBackground.a)

	
	--dragFrame dragging.
	dragFrame.mouseStartX = 0
	dragFrame.mouseStartY = 0
	dragFrame.mouseIsDown = false
	--Mouse move handler.
	dragFrame:EventAttach(Event.UI.Input.Mouse.Cursor.Move, 
		function (self)
			if self.mouseIsDown and not barTableRef.locked and not Inspect.System.Secure() then
				barTableRef.xOffset = barTableRef.xOffset + Inspect.Mouse().x - self.mouseStartX
				barTableRef.yOffset = barTableRef.yOffset + Inspect.Mouse().y - self.mouseStartY
				self.mouseStartX = Inspect.Mouse().x
				self.mouseStartY = Inspect.Mouse().y
				self:SetPoint("CENTER", UIParent, "CENTER", barTableRef.xOffset, barTableRef.yOffset)
			end
		end, 
	dragFrame:GetName().." cursor move event")
	--Left mouse down handler.
	dragFrame:EventAttach(Event.UI.Input.Mouse.Left.Down, 
		function (self)
			self.mouseIsDown = true
			self.mouseStartX = Inspect.Mouse().x
			self.mouseStartY = Inspect.Mouse().y
		end, 
	dragFrame:GetName().." left mouse down event")
	--Left mouse up handler.
	dragFrame:EventAttach(Event.UI.Input.Mouse.Left.Up,
		function (self)
			self.mouseIsDown = false
		end,
	dragFrame:GetName().." left mouse up event")
	--Left mouse up-outside handler.
	dragFrame:EventAttach(Event.UI.Input.Mouse.Left.Upoutside,
		function (self)
			self.mouseIsDown = false
		end,
	dragFrame:GetName().." left mouse up-outside event")
	
	dragFrame:SetSecureMode("restricted")
	
	--Delete frame button. Below the drag frame button.
	local deleteFrameButton = UI.CreateFrame("Frame", barFrameOuter:GetName().."_deleteFrameButton"..barFrameNumber, barFrameOuter)
	deleteFrameButton:SetLayer(10)
	deleteFrameButton:SetWidth(barTableRef.deleteFrameButtonWidth)
	deleteFrameButton:SetHeight(barTableRef.deleteFrameButtonHeight)
	deleteFrameButton:SetPoint("TOPLEFT", dragFrame, "BOTTOMLEFT")
	deleteFrameButton:SetBackgroundColor(barTableRef.deleteFrameButtonBackground.r,
								barTableRef.deleteFrameButtonBackground.g,
								barTableRef.deleteFrameButtonBackground.b,
								barTableRef.deleteFrameButtonBackground.a)
	
	--
	--Left mouse down handler.
	deleteFrameButton:EventAttach(Event.UI.Input.Mouse.Left.Down, 
		function (self)
			self.mouseIsDown = true
		end, 
	deleteFrameButton:GetName().." left mouse down event")
	--Left mouse up handler.
	deleteFrameButton:EventAttach(Event.UI.Input.Mouse.Left.Up,
		function (self)
			if (self.mouseIsDown) then
				--Prompt the user as to whether or not they want to delete the frame.
				
				--local reference to self, for use inside the button functions.
				local this = self
				
				if not (self.promptExists) then
					self.promptExists = true
					--Create a riftwindow, with two rift buttons: "Delete" and "Cancel".
					self.prompt = UI.CreateFrame("RiftWindow", self:GetName().."_prompt", self)
					self.prompt:SetTitle("Delete frame?")
					self.prompt:SetPoint("CENTER", UIParent, "CENTER")
					self.prompt:SetHeight(80)
					
					
					local affirmativeButton = UI.CreateFrame("RiftButton", self.prompt:GetName().."_affirmativeButton", self.prompt)
					affirmativeButton:SetText("Delete")
					affirmativeButton:SetPoint("BOTTOMLEFT", self.prompt, "BOTTOMLEFT", 10, 0)
					
					--Left mouse up handler.
					affirmativeButton:EventAttach(Event.UI.Input.Mouse.Left.Up,
						function (self)
							--Close prompt.
							this.prompt:SetVisible(false)
							--Hide bar.
							barFrameOuter:SetVisible(false)
							--Delete data.
							rust.settings.actionBar.action_bars[currentProfile][barFrameNumber] = nil
						end,
					affirmativeButton:GetName().." left mouse up event")
					
					local negativeButton = UI.CreateFrame("RiftButton", self.prompt:GetName().."_negativeButton", self.prompt)
					negativeButton:SetText("Cancel")
					negativeButton:SetPoint("BOTTOMRIGHT", self.prompt, "BOTTOMRIGHT", -10, 0)
					
					--Left mouse up handler.
					negativeButton:EventAttach(Event.UI.Input.Mouse.Left.Up,
						function (self)
							--Close prompt.
							this.prompt:SetVisible(false)
						end,
					negativeButton:GetName().." left mouse up event")
				else
					--If the prompt already exists, we make it visible.
					self.prompt:SetVisible(true)
				end
			end
		
			self.mouseIsDown = false
		end,
	deleteFrameButton:GetName().." left mouse up event")
	--Left mouse up-outside handler.
	deleteFrameButton:EventAttach(Event.UI.Input.Mouse.Left.Upoutside,
		function (self)
			self.mouseIsDown = false
		end,
	deleteFrameButton:GetName().." left mouse up-outside event")
	
	--Create our frame to anchor individual buttons to.
	local barFrameInner = UI.CreateFrame("Frame", barFrameOuter:GetName().."_barFrameInner", barFrameOuter)
	barFrameInner:SetPoint("TOPLEFT", barFrameInner:GetParent(), "TOPLEFT", barTableRef.horizontalEdgePadding, barTableRef.verticalEdgePadding)
	barFrameInner:SetBackgroundColor(barTableRef.barInnerBackground.r,
								barTableRef.barInnerBackground.g,
								barTableRef.barInnerBackground.b,
								barTableRef.barInnerBackground.a)
	
	barFrameInner:SetWidth(barFrameOuter:GetWidth() - (barTableRef.horizontalEdgePadding * 2))
	barFrameInner:SetHeight(barFrameOuter:GetHeight() - (barTableRef.verticalEdgePadding * 2))
	
	barFrameInner:SetSecureMode("restricted")
	
	
	--Create a table to store button information in.
	barTableRef.buttons = barTableRef.buttons or {}
	barTableRef.buttons[playerName] = barTableRef.buttons[playerName] or {}
	useString = "cast" --The keyword for casting/using/equiping the desired "thing" in a macro.
	--Create ability buttons.
	for i = 1, (barTableRef.rows * barTableRef.columns) do
	
		--Create button.
		--abilityNumber = abilityNumber + 1
		local buttonBG = UI.CreateFrame("Texture", barFrameInner:GetName().."_buttonBG_"..i, barFrameInner)
		local button = UI.CreateFrame("Texture", buttonBG:GetName().."_button_"..i, buttonBG)
		
		--Size.
		buttonBG:SetWidth(barTableRef.width)
		buttonBG:SetHeight(barTableRef.height)
		button:SetWidth(buttonBG:GetWidth())
		button:SetHeight(buttonBG:GetHeight())
		
		--Location.
		
		--TODO bar growth
		--store the anchor (topleft, topright) in a variable. We derive it from the direction that the user wants the bar to grow in - right, then down -> topleft (opposite, with vertical then horizontal)
		--a positive X offset means offset to the right. negative -> left.
		--for Y offset, positive -> down, negative -> up
		--so, after calculating the raw offset (positive value only), we multiply by negative one (-1) if needed.
		--grow to the left -> multiply by -1
		--grow upwards -> multiply by -1
		
		local buttonNumber = i - 1
		local buttonXOffset = (barTableRef.width + barTableRef.horizontalSpacing) * (buttonNumber % barTableRef.columns)
		
		local buttonYOffset = (barTableRef.height + barTableRef.verticalSpacing) * math.floor(buttonNumber / barTableRef.columns)
		
		buttonBG:SetPoint("TOPLEFT", buttonBG:GetParent(), "TOPLEFT", buttonXOffset, buttonYOffset)
		button:SetPoint("CENTER", button:GetParent(), "CENTER")
		
		--Background colour.
		buttonBG:SetBackgroundColor(barTableRef.buttonBackground.r,
								barTableRef.buttonBackground.g,
								barTableRef.buttonBackground.b,
								barTableRef.buttonBackground.a)
		buttonBG:SetSecureMode("restricted")

		--Functionality.
		--Mouseover.
		--Cursor in.
		button:EventAttach(Event.UI.Input.Mouse.Cursor.In, 
			function (self)
				Command.Tooltip(self.id)
			end,
		button:GetName().." cursor in event")
		--Cursor out.
		button:EventAttach(Event.UI.Input.Mouse.Cursor.Out, 
			function (self)
				--Dragging content from button related.
				--Don't allow moving abilities/items in combat.
				if not (Inspect.System.Secure()) then
					if (Inspect.Mouse().Left == true) and (barFrameInner.clickedButton == self) then
						--We place the current button contents on the cursor, and clear the button.
						--Command.Cursor(self.id)
						--Command.Item.Standard.Drag(self.id)
						--self:clear()
						return
					end
				end
				
				
				--Tooltip related.
				local tooltipType, tooltipUnitOrShown, tooltipBuff = Inspect.Tooltip()
				if ((tooltipUnitOrShown == self.id) or (tooltipBuff and (tooltipBuff == self.id))) then
					--We only clear the tooltip if the currently shown tooltip is ours.
					--If you mouseover between two buttons quickly, it sometimes fires: cursor.in(button1), cursor.in(button2), cursor.out(button1).
					Command.Tooltip(nil)
				end
			end,
		button:GetName().." cursor in event")
		
		function button:placeAbilityOrItem(buttonType, buttonID)
			local oldButtonID = self.id
			
			local validAbilityOrItem = false
			local abilityItemBlob = nil
			
			if buttonType == "ability" then
				if Inspect.Ability.New.List()[buttonID] then --Do we know this ability?
					abilityItemBlob = Inspect.Ability.New.Detail(buttonID)
					
					validAbilityOrItem = true
					
					--Getting the correct useString.
					useString = "cast"
				end
			elseif buttonType == "item" then
				abilityItemBlob = Inspect.Item.Detail(buttonID)
				if abilityItemBlob then --Do we know anything about this item?
			
					validAbilityOrItem = true
			
					--Getting the correct useString.
					local itemCategoryString = abilityItemBlob.category
					itemCategoryIt = string.gmatch(itemCategoryString, "%S+")
					itemCategoryString = itemCategoryIt()
					if (itemCategoryString == "armor") or (itemCategoryString == "weapon") or (itemCategoryString == "planar") then
						useString = "equip"
					else
						useString = "use"
					end
					
					Command.Cursor(nil)
				end
			else
				return
			end
			
			if (validAbilityOrItem) then
				self.id = buttonID
				self:EventMacroSet(Event.UI.Input.Mouse.Left.Click, useString .. " "..abilityItemBlob.name)
				self:SetTexture("Rift", abilityItemBlob.icon)
			else
				--Backup icon.
				self:SetTexture("Rift", barTableRef.icon)
			end
			
			--Put the old ability/item on the cursor, to simulate "swapping".
			--[[if (oldButtonID) then]]
			--Command.Cursor(oldButtonID)
			--[[else
				Command.Cursor(nil)
			end
			print(oldButtonID)]]--
		end
		
		
		--Left mouse down.
		button:EventAttach(Event.UI.Input.Mouse.Left.Down, 
			function (self)
				--This tracks which button we initially mouse-down on.
				barFrameInner.clickedButton = self
			end,
		button:GetName().." left mouse down")
		
		--Left mouse release.
		button:EventAttach(Event.UI.Input.Mouse.Left.Up, 
			function (self)
				barFrameInner.clickedButton = nil
				--Don't allow placing abilities in combat.
				if Inspect.System.Secure() then return end
			
				local cursorType, cursorHeld = Inspect.Cursor()
				if cursorType and cursorHeld then
				
					self:placeAbilityOrItem(cursorType, cursorHeld)
					
					self:SetAlpha(1)
					
					barTableRef.buttons[playerName][i] = {}
					barTableRef.buttons[playerName][i]["id"] = cursorHeld
					barTableRef.buttons[playerName][i]["type"] = cursorType
					
					--Command.Tooltip(cursorHeld)
					--Command.Cursor(cursorHeld)
				end
			end,
		button:GetName().." left mouse release")
		--Right click.
		button:EventAttach(Event.UI.Input.Mouse.Right.Click, 
			function (self)
				self:clear()
				
				Command.Tooltip(nil)
			end,
		button:GetName().." right mouse release")
		
		function button:clear()
			if Inspect.System.Secure() then return end
			
			self.id = nil
			self:SetAlpha(0)
			self:EventMacroSet(Event.UI.Input.Mouse.Left.Click, nil)
			
			barTableRef.buttons[playerName][i] = {}
		end
		
		--Secure mode (allows :EventMacroSet(...)).
		button:SetSecureMode("restricted")
		
		--Restore a previously saved action.
		if barTableRef.buttons[playerName] and barTableRef.buttons[playerName][i] then
			local buttonType = barTableRef.buttons[playerName][i]["type"]
			local buttonID = barTableRef.buttons[playerName][i]["id"]
			button:placeAbilityOrItem(buttonType, buttonID)
		end
		
		
		
		--[[
		--Multi-line macro example.
		button:EventMacroSet(Event.UI.Input.Mouse.Left.Click, "#show keen strike\
																\nsuppressmacrofailures\
																\ncast @mouseover reprisal\
																\ncast @mouseover quick strike\
																\ncast @mouseover keen strike")
		]]--
		--Click.
		--button:EventMacroSet(Event.UI.Input.Mouse.Left.Click, "cast "..Inspect.Ability.New.Detail(k).name)
	end
	
	--barFrame:SetWidth(previousButton:GetRight() + dragFrame.settings.horizontalEdgePadding - barFrame:GetLeft())
	--barFrame:SetHeight(dragFrame.settings.height + (2 * dragFrame.settings.verticalEdgePadding))
	
	return barFrame
end

local function restoreSavedBars()
	if rust.settings.actionBar.action_bars[currentProfile] then
		for i = 1, table.getn(rust.settings.actionBar.action_bars[currentProfile]) do
			--print(i) -- temp
			createBar()
		end
	end
end

local function slashHandler(a, message)
	--Separate the 'message' into parts.
	local messageParts = {}
	for w in string.gmatch(message, "%S+") do
		table.insert(messageParts, w)
	end
	
	
	if messageParts[1] == "create" then
		local param1 = tonumber(messageParts[2])
		local param2 = tonumber(messageParts[3])
		if type(param1) == "number" and type(param2) == "number" then
			createBar(param1, param2)
		end
	end
end
	
local function main()
	--Restore saved bars.
	
	--Check that the current player has a profile mapped.
	if not (rust.mappings[playerName]) then
		--If not, we create one for them.
		rust.mappings[playerName] = playerName
	end
	--Set the file-scope profile variable.
	currentProfile = rust.mappings[playerName]
	
	--Check that the current profile has an action bar table.
	rust.settings.actionBar.action_bars[currentProfile] = rust.settings.actionBar.action_bars[currentProfile] or {}
	
	--We do this to avoid causng errors if the user logs in/reloads their UI while in combat.
	if not Inspect.System.Secure() then
		restoreSavedBars()
	else
		Command.Event.Attach(Event.System.Secure.Leave,
			function (self)
				restoreSavedBars()
				Command.Event.Detach(Event.System.Secure.Leave, nil, "System secure leave (restore saved bars) handler")
			end,
		"System secure leave (restore saved bars) handler")
	end
	
	--Slash commands.
	Command.Event.Attach(Command.Slash.Register("rustactionbars"), 
						slashHandler, addonName)
	Command.Event.Attach(Command.Slash.Register("rab"), 
						slashHandler, addonName)
	
	--
	

	--todo
	--TODO have the option of having bar profiles set per soul role (e.g., role 1 = profile A, role 2 = profile B).
	--TODO same as above, except with button contents.
	--TODO cooldowns don't display on buttons.
	--TODO allow the contents of buttons to be dragged off.
	--each bar should be draggable and lockable.
	--should be able to rearrange bars, even move spells between them.
	--should be hidable. Maybe allow for hiding in combat.
	--Consider some way of shrinking their appearance (think of WoW-mage teleport to... buttons).
	--create a config window, to allow for configuration.


	--remove this if not needed - it was some unrelated testing.
	--Register events.
	--Command.Event.Attach(Event.System.Update.Begin, update, "system.update.begin handler")
	
end

--We must wait until player abilites are available before restoring old frames.
Command.Event.Attach(Event.Ability.New.Add,
	function (self, abilities)
		if not hasAlreadyLoaded then
			main()
			hasAlreadyLoaded = true
		end
	end, 
"Ability add handler")

--Handler for loading saved variables.
Command.Event.Attach(Event.Addon.SavedVariables.Load.End,
	function (self, addonIdentifier)
		if (addonIdentifier == Inspect.Addon.Current()) then
			if (rusty_ActionBars_account) then
				--Saved variable exists.
				rust = rusty_ActionBars_account -- TODO re-enable loading saved vars when saved var table is complete.
			end
		end
	end, 
"Saved variables load end handler")

--Handler for saving saved variables.
Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, 
	function (self, addonIdentifier)
		rusty_ActionBars_account = rust
	end, 
"Saved variables save begin handler")