------------------------------------------------------------------
------------------ Super Meter - Event Handlers ------------------
------------------------------------------------------------------

function SuperMeter.Blocked(params)
	if not SuperMeter_enabled or not params then return end

	local instance = SuperMeter_instance
	local inspect = Inspect.Unit.Detail
	local range = SuperMeter.Range
	local sync = SuperMeter.Sync
	local format = tostring

	for k,v in pairs(params) do
		local unit = inspect(format(k))
		if not range[k] and sync[k] and instance and unit and unit.blocked then
			range[k] = true
		end
		if range[k] and unit and not unit.blocked then
			range[k] = nil
		end
	end
end

function SuperMeter.Combat(params)
	if not SuperMeter_enabled then return end

	local session = SuperMeter.Session(false, true)
	local pet = Inspect.Unit.Lookup("player.pet")
	local performance = SuperMeter_performance
	local current = SuperMeter.Session(true)
	local combat = SuperMeter.Player.combat
	local interface = SuperMeter.Interface
	local showing = SuperMeter_logShowing
	local instance = SuperMeter_instance
	local sessions = SuperMeter.Sessions
	local filters = SuperMeter.Filters()
	local inspect = Inspect.Unit.Detail
	local inCombat = SuperMeter_combat
	local send = Command.Message.Send
	local battle = SuperMeter.Battle
	local forced = SuperMeter_forced
	local player = inspect("player")
	local time = Inspect.Time.Real()
	local warfront = SuperMeter_pvp
	local debug = SuperMeter.Debug
	local check = SuperMeter_check
	local force = SuperMeter_force
	local group = SuperMeter_group
	local merge = SuperMeter_merge
	local split = SuperMeter_split
	local entry = SuperMeter.Unit
	local format = string.format
	local text = player.syncName

	if not params then
		params = {[player.id] = not not player.combat}
	end

	if params[player.id] then
		text = format("<syncable> <%0s>", player.id)
	end

	if not instance and player.zone then
		local zone = Inspect.Zone.Detail(player.zone)
		
		if zone and zone.type == "instance" then
			SuperMeter_instance = true
			instance = true
		end
		
		for k,v in pairs(params) do
			if group and not instance then
				local unit = inspect(format(k))
				if unit and unit.tier == "raid" and not filters.tier[unit.name] then
					SuperMeter_instance = true
					instance = true
				end
			end
		end
	end

	for k,v in pairs(params) do
		if group and instance or k == player.id or k == pet then
			battle[k] = not not v
		end
	end

	for k,v in pairs(battle) do
		local unit = inspect(format(k))
		if forced or unit and unit.combat or unit and unit.availability == "partial" and v then
			if text and unit and unit.combat and unit.player and unit.name and not (k == player.id) then send(unit.name, "sync", text, debug) end
			combat = true
		end
	end

	if not combat and current.combatTime <= 2 then
		table.remove(sessions, #sessions)
		interface.ResetTime()
	end

	if not force and not forced then
		SuperMeter.Player.combat = player.combat
	else
		SuperMeter.Player.combat = true
	end

	if instance and not performance then
		SuperMeter_pvp = false
		player.pvp = false
		warfront = false
	end

	if combat or not player.combat then
		SuperMeter.Blocked(battle)
	end

	if split and inCombat then
		SuperMeter_combat = false
		SuperMeter_split = false
		current.split = true
		inCombat = false
	end

	if force then
		SuperMeter_force = false
		combat = not inCombat
	end

	if not combat then
		SuperMeter.Battle = {}
		SuperMeter.Ignore = {}
		SuperMeter.Range = {}
		SuperMeter.Sync = {}
	end

	if combat and not inCombat then
		session.startCombat = time
		SuperMeter_combat = time
		SuperMeter.Refresh = 0
		SuperMeter.Offset = 0
		
		if current and current.merge then
			table.remove(sessions, #sessions)
			interface.ResetTime()
		end
		
		if sessions and session then
			table.insert(sessions, session)
			interface.ResetTime()
		end
		
		if #sessions > 200 then
			table.remove(sessions, 1)
		end
		
		interface.Manager()
		interface.Change()
	end

	for k,v in pairs(params) do
		local units = {caster = k, target = k}
		if combat and not force and v then
			entry(units, "caster")
		end
	end

	if not combat and inCombat then
		SuperMeter_instance = false
		SuperMeter_combat = false
		SuperMeter_forced = false
		SuperMeter_boss = false
		
		if current and merge then
			SuperMeter.Merge()
		end
		
		interface.Manager()
		interface.Change()
	end

	if player.pvp and not warfront then
		SuperMeter.Warfront(true)
	end

	if not combat and not showing then
		interface.Combat(true)
	end

	if not check and not showing then
		interface.Combat(true)
	end

	if #sessions > 0 and time then
		SuperMeter_check = false
		SuperMeter.Timer = time
	end
end

function SuperMeter.Damage(params)
	if not SuperMeter_enabled or not params then return end

	local gather = (SuperMeter.Gather or SuperMeter.Debug)
	local unit = (SuperMeter.Unit or SuperMeter.Debug)

	local caster = unit(params, "caster")
	local target = unit(params, "target")

	if not caster or not target then return end

	gather(params, "damage", caster, target)
end

function SuperMeter.Dodge(params)
	if not SuperMeter_enabled or not params then return end

	local gather = (SuperMeter.Gather or SuperMeter.Debug)
	local unit = (SuperMeter.Unit or SuperMeter.Debug)

	local caster = unit(params, "caster")
	local target = unit(params, "target")

	if not caster or not target then return end

	gather(params, "avoid", caster, target)
end

function SuperMeter.Enter(params)
	if not SuperMeter_enabled then return end

	if not SuperMeter_combat then
		local inspect = Inspect.Unit.Detail
		local list = Inspect.Unit.List()
		local format = tostring
		
		local units = {}
		
		for k,v in pairs(list) do
			local detail = inspect(format(k))
			if detail and detail.health and detail.health > 0 then
				units[k] = not not detail.combat
			end
		end
		
		SuperMeter.Combat(units)
	end
end

function SuperMeter.Exit(params)
	if not SuperMeter_enabled then return end

	if SuperMeter_combat then
		local inspect = Inspect.Unit.Detail
		local list = Inspect.Unit.List()
		local format = tostring
		
		local units = {}
		
		for k,v in pairs(list) do
			local detail = inspect(format(k))
			if detail and detail.level and detail.name then
				units[k] = not not detail.combat
			end
		end
		
		SuperMeter.Combat(units)
	end
end

function SuperMeter.Heal(params)
	if not SuperMeter_enabled or not params then return end

	local gather = (SuperMeter.Gather or SuperMeter.Debug)
	local unit = (SuperMeter.Unit or SuperMeter.Debug)

	local caster = unit(params, "caster")
	local target = unit(params, "target")

	if not caster or not target then return end

	gather(params, "heal", caster, target)
end

function SuperMeter.Miss(params)
	if not SuperMeter_enabled or not params then return end

	local gather = (SuperMeter.Gather or SuperMeter.Debug)
	local unit = (SuperMeter.Unit or SuperMeter.Debug)

	local caster = unit(params, "caster")
	local target = unit(params, "target")

	if not caster or not target then return end

	gather(params, "miss", caster, target)
end

function SuperMeter.Parry(params)
	if not SuperMeter_enabled or not params then return end

	local gather = (SuperMeter.Gather or SuperMeter.Debug)
	local unit = (SuperMeter.Unit or SuperMeter.Debug)

	local caster = unit(params, "caster")
	local target = unit(params, "target")

	if not caster or not target then return end

	gather(params, "avoid", caster, target)
end

function SuperMeter.Receive(from, type, channel, identifier, text)
	if not SuperMeter_enabled or not SuperMeter_instance then return end
	if not (identifier == "sync") then return end

	local events = {damage = 0, damageAbsorbed = 0, damageAvoided = 0, damageBlocked = 0, damageDeflected = 0, damageIntercepted = 0, damageModified = 0, heal = 0, overheal = 0, overkill = 0}

	local dummy = SuperMeter.Unit(events, "caster")
	local session = SuperMeter.Session(true)
	local gather = SuperMeter.Gather

	local format = string.format
	local match = string.match
	local find = string.find
	local integer = tonumber
	local sub = string.gsub

	local caster = false
	local target = false
	local event = false

	local data = {}

	for k,v in pairs(events) do
		data[k] = integer(match(text, format("<%0s> <(%%d+)>", k)))
	end

	if not data.abilityName and text then
		data.abilityName = match(text, "<abilityName> <(.-)>")
		data.abilityNew = match(text, "<abilityNew> <(.-)>")
		data.casterName = match(text, "<casterName> <(.-)>")
		data.targetName = match(text, "<targetName> <(.-)>")
		data.caster = match(text, "<caster> <(.-)>")
		data.target = match(text, "<target> <(.-)>")
		data.type = match(text, "<type> <(.-)>")
	end

	if data.abilityName and text then
		data.crit = match(text, "<crit> <(true)>")
		data.sync = not not next(data)
	end

	if not event and text then
		caster = (session.units[data.caster] or dummy)
		target = (session.units[data.target] or dummy)
		event = match(text, "<event> <(.-)>")
	end

	if caster and target then
		local unit = match(text, "<syncable> <(.-)>")
		if unit and not SuperMeter.Sync[unit] then
			SuperMeter.Sync[unit] = 0
		end
	end

	if caster and target then
		gather(data, event, caster, target)
	end
end

function SuperMeter.Resist(params)
	if not SuperMeter_enabled or not params then return end

	local gather = (SuperMeter.Gather or SuperMeter.Debug)
	local unit = (SuperMeter.Unit or SuperMeter.Debug)

	local caster = unit(params, "caster")
	local target = unit(params, "target")

	if not caster or not target then return end

	gather(params, "avoid", caster, target)
end

function SuperMeter.Update(params)
	if not SuperMeter_enabled then return end
	if not SuperMeter_combat then return end

	local session = SuperMeter.Session(true)
	local interface = SuperMeter.Interface
	local refresh = SuperMeter.Refresh
	local forced = SuperMeter_forced
	local player = SuperMeter.Player
	local time = Inspect.Time.Real()
	local timer = SuperMeter.Timer

	if session and time then
		session.combatTime = time - session.startCombat
	end

	if not player.combat and time - timer >= 0.82 then
		SuperMeter_check = true
		SuperMeter.Combat()
	end

	if session and time - refresh >= 0.98 then
		SuperMeter.Refresh = time
		interface.Update(2)
		interface.Time()
	end

	if forced and time - timer >= 0.42 then
		SuperMeter_forced = false
		SuperMeter.Combat()
	end
end

function SuperMeter.Warfront(params)
	if not SuperMeter_enabled or not params then return end

	local player = Inspect.Unit.Detail("player")
	local interface = SuperMeter.Interface
	local warfront = SuperMeter_pvp

	if player.pvp and not warfront then
		print(SuperMeter_region21)
		SuperMeter_pvp = true
	end

	SuperMeter_pvp = player.pvp
	interface.Display()
end

table.insert(Event.Combat.Damage, {SuperMeter.Damage, "SuperMeter", "Combat Damage"})
table.insert(Event.Combat.Dodge, {SuperMeter.Dodge, "SuperMeter", "Combat Dodges"})
table.insert(Event.Combat.Heal, {SuperMeter.Heal, "SuperMeter", "Combat Heals"})
table.insert(Event.Combat.Miss, {SuperMeter.Miss, "SuperMeter", "Combat Misses"})
table.insert(Event.Combat.Parry, {SuperMeter.Parry, "SuperMeter", "Combat Parries"})
table.insert(Event.Message.Receive, {SuperMeter.Receive, "SuperMeter", "Addon Sync"})
table.insert(Event.Combat.Resist, {SuperMeter.Resist, "SuperMeter", "Combat Resists"})
table.insert(Event.System.Update.Begin, {SuperMeter.Update, "SuperMeter", "Update UI"})
table.insert(Event.System.Update.End, {SuperMeter.Update, "SuperMeter", "Update UI"})
table.insert(Event.System.Secure.Enter, {SuperMeter.Enter, "SuperMeter", "Secure Enter"})
table.insert(Event.System.Secure.Leave, {SuperMeter.Exit, "SuperMeter", "Secure Exit"})
table.insert(Event.Unit.Detail.Blocked, {SuperMeter.Blocked, "SuperMeter", "Unit Blocked"})
table.insert(Event.Unit.Detail.Combat, {SuperMeter.Combat, "SuperMeter", "Unit Combat"})
table.insert(Event.Unit.Detail.Pvp, {SuperMeter.Warfront, "SuperMeter", "Unit PvP"})